% Sampling change
% [xxr,ttr,fxmin,fxmax,step]=sampling_change(xx,tt,newsampling,ftol,method,extrapval) ;
%
% Script to interpolate a signal xx(tt) over a new sampling.
% 
% INPUT	xx = signal (must be real)
%	tt = original time vector
%	newsampling = a vector of the new smpling times
%	..or..  'a' arithmetical sampling
%		'g' geometrical sampling 
%		'l' logarithmic sampling
%	ftol = 0 to have great tolerance with Shannon bound (default)
%	     = 1 to have no trespassing of Shannon bound
%	     = 2 to use the frequency content of xx
%            = a numer between 0 and 1 (excluded) to use the frequency content 
%	of xx whith this number = treshhold to estimate highest frequency (default 0.01)
%	method = method in interp1 (default 'linear')
%	etrapval = extrapolation for xxr (default NaN)
%		(see help of interp1 for these two parameters)
%
% OUTPUT xxr = re-sampled signal
%	ttr = new sampling vector
%	fxmin = minimal Shannon bound of xx
%	fxmax = maximal Shannon bound of xx
%       step = ratio of the series
%
% Warning: because of the need to estimate the frequency content of xx,
%(potentially unenvenly sampled, by Lomb periodogram), the conversion might 
%be long if one does not give the new sampling vector and use ftol~=0 or 1 
%
% (pborgnat 2002)
 

function [xxr,ttr,fxmin,fxmax,step]=sampling_change(xx,tt,newsampling,ftol,method,extrapval) ;

xx=real(xx);

tolmeth=( (ftol~=0)&(ftol~=1) );
if tolmeth,
tresh=ftol;
if tresh>1, tresh=0.01; end;
end;

if nargin<4, ftol=0; tresh=0.01; end;
if nargin<5, method='linear'; end;
if nargin<6, extrapval=NaN; end

if newsampling=='a',

dt=diff(tt);
fmax=0.5/min(dt);
fmin=0.5/max(dt);
if ftol==1,  ftol=fmin;
	else ftol=fmax; 
end;
Tmax=max(tt);
%N=length(xx);
if tolmeth,
	[per,fx]=per_lomb(tt,xx,4,1.1) ;
	seuil=tresh*max(per) ;
	[blah,jj]=find(per>seuil);
	fe=min(2*fx(max(jj)),ftol);
else,   fe=ftol;
end;
ttr=0:(1/fe):Tmax;
step=1/fe;

elseif newsampling=='g',

[yy,uu] = lamperti_invtrans(xx,0,tt) ;
ii=find(isfinite(uu));
dt=diff(uui(ii));
fmax=0.5/min(dt);
fmin=0.5/max(dt);
if ftol==1,  ftol=fmin;
	else ftol=fmax; 
end;
Tmax=max(tt);
%N=length(yy);
if tolmeth,
	[per,fx]=per_lomb(uu(ii),yy(ii),4,1.1) ;
	seuil=tresh*max(per) ;
	[blah,jj]=find(per>seuil);
	bmax=min(2*fx(max(jj)),ftol);
else, bmax=ftol;
end;
lnq=1/bmax;
%q=exp(lnq);
Nr=round(log(Tmax)/lnq);
ttr=exp(lnq*(0:Nr));
step=exp(lnq);

elseif newsampling=='l',

[xxt,uu] = lamperti_trans(xx,0,tt) ;
ii=find(isfinite(uu));
dt=diff(uu(ii));
fmax=0.5/min(dt);
fmin=0.5/max(dt);
if ftol==1,  ftol=fmin;
	else ftol=fmax;
end;
Tmax=max(tt);
if tolmeth,
	[per,fx]=per_lomb(uu(ii),xxt(ii),4,1.1) ;
	seuil=tresh*max(per) ;
	[blah,jj]=find(per>seuil);
	fe=min(2*fx(max(jj)),ftol);
else,   fe=ftol;
end;
Nr=min(exp(Tmax)*fe,10^10);
ttr=log((1:Nr)/fe);
step=1/fe;

else,
ttr=newsampling;

end;

xxr=interp1(tt,xx,ttr,method,extrapval);

dt=diff(tt);
fxmax=0.5/min(dt);     
fxmin=0.5/max(dt);

